#!/usr/bin/env python
import wx, os, ConfigParser
from dialogs import IODialog

def init_dplug(dplug):
    dplug.plugins.customize = Customize(dplug)
    dplug.add_to("new", "customize")
    dplug.add_to("open", "customize")
    dplug.add_to("configsave", "customize")
    dplug.add_to("configinit", "customize")
    dplug.add_cmd("theme", "customize")

def trigger(trig, dplug):
    if trig == "configsave":
        dplug.plugins.customize.config.Update()
    elif trig == "configinit":
        dplug.plugins.customize.config.Init()
    else:
        dplug.plugins.customize.theme()

def command(cmd, next, i):
    if cmd == "theme":
        i.plugin_config.set("customize", "theme", next + ".ded")
        i.plugins.customize.theme()

class Customize():
    def __init__(self, dplug):
        self.dplug = dplug
        self.gui = dplug.main_class
        
        self.dplug.plugin_config.add_section("customize")
        if not self.dplug.plugin_config.get("customize","theme"): self.dplug.plugin_config.set("customize","theme","default")
        
        self.theme()
        
        self.config = Config(self.gui.confpan, -1, dplug = self.dplug)
    
    def theme(self):
        theme = self.dplug.plugin_config.get("customize", "theme")
        if theme != "default" and os.path.isfile(os.path.join(self.dplug.plugindir, "Themes", theme)):
            cf = os.path.join(self.dplug.plugindir, "Themes", theme)
            config = ConfigParser.ConfigParser()
            config.read(cf)
            
            for i in config.options("foreground"):
                self.set_foreground(i, config.get("foreground", i))
            self.set_general_background(config.get("general","background"))
            for i in config.options("background"):
                self.set_background(i, config.get("background", i))
            self.set_font(config.get("general", "font"), int(config.get("general", "size")))
        if self.dplug.plugin_config.get("customize", "overwrite") == "True":
            font = self.dplug.plugin_config.get("customize", "font")
            size = self.dplug.plugin_config.get("customize", "size")
            self.set_font(font, int(size))
            
    def set_font(self, name, size):
        for tab in self.gui.tabs:
            if tab.isFile:
                for i in range(0,33):
                    tab.editor.StyleSetFontAttr(i, size, name, 0, 0, 0)

    def set_background(self, option, colour):
        for tab in self.gui.tabs:
            if tab.isFile:
                tab.editor.StyleSetBackground(getattr(wx.stc, option.upper()), colour)
    
    def set_foreground(self, option, colour):
        for tab in self.gui.tabs:
            if tab.isFile:
                tab.editor.StyleSetForeground(getattr(wx.stc, option.upper()), colour)
        
    def set_general_background(self, colour):
        for tab in self.gui.tabs:
            if tab.isFile:
                for i in range(0,33):
                    tab.editor.StyleSetBackground(i, colour)

class Config(wx.Panel):
    def __init__(self, parent, sth, dplug):
        self.dplug = dplug
        self.gui = dplug.main_class
        wx.Panel.__init__(self, parent, sth)
        self.siz = wx.GridSizer(3, 2, 0, 0)
        self.siz.Add(wx.StaticText(self, -1, "Theme: "))
        self.themes = []
        self.theme = wx.ComboBox(self, -1, size=(150, -1), choices = self.themes, style = wx.CB_READONLY)
        self.font = wx.FontPickerCtrl(self)
        self.siz.Add(self.theme)
        self.loadbut = wx.Button(self, -1, "Load")
        self.siz.Add((20, 20), -1, wx.EXPAND)
        self.siz.Add(self.loadbut)
        self.siz.Add(wx.StaticText(self, -1, "Font: "))
        self.siz.Add(self.font)
        self.siz.Add((20, 20), -1, wx.EXPAND)
        self.overwrite = wx.CheckBox(self, -1, "Overwrite theme font")
        self.siz.Add(self.overwrite)
        self.loadbut.Bind(wx.EVT_BUTTON, lambda event: self.load())
        #self.mod = wx.Button(self, -1, "Test Colours")
        #self.mod.Bind(wx.EVT_BUTTON, lambda event: self.modify())
        #self.siz.Add(self.mod)
        self.SetSizer(self.siz)
        self.button_name = "Interface"
        parent.plugin_add(self, expand=False)
    
    def modify(self):
        Fore(None,-1, "", i = self.dplug)
    
    def load(self):
        if self.theme.GetSelection() == 0:
            for tab in self.gui.tabs:
                if tab.isFile:
                    tab.editor.StyleResetDefault()
                    tab.editor.StyleClearAll()
            self.dplug.plugin_config.set("customize","theme", "default")
        else:
            self.dplug.plugin_config.set("customize", "theme", self.themes[self.theme.GetSelection()] + ".ded")
            self.dplug.plugins.customize.theme()
        
    def Init(self):
        i = 1
        self.themes = []
        self.themes.append("default")
        for theme in os.listdir(os.path.join(self.dplug.plugindir, "Themes")):
            if theme == self.dplug.plugin_config.get("customize", "theme"): j = i
            self.themes.append(theme[:-4])
            i += 1
        self.theme.SetItems(self.themes)
        try:
            self.theme.SetSelection(j)
        except UnboundLocalError:
            self.theme.SetSelection(0)
        font = self.dplug.plugin_config.get("customize", "font")
        size = self.dplug.plugin_config.get("customize", "size")
        overwrite = self.dplug.plugin_config.get("customize", "overwrite")
        if overwrite == "True": self.overwrite.SetValue(1)
        else: self.overwrite.SetValue(0)
        custom_font = self.font.GetSelectedFont()
        custom_font.SetFaceName(font)
        custom_font.SetPointSize(int(size))
        self.font.SetSelectedFont(custom_font)

    def Update(self):
        f = self.font.GetSelectedFont()
        self.dplug.plugin_config.set("customize", "font", f.GetFaceName())
        self.dplug.plugin_config.set("customize", "size", str(f.GetPointSize()))
        if self.overwrite.GetValue() == 1:  self.dplug.plugin_config.set("customize", "overwrite", "True")
        else: self.dplug.plugin_config.set("customize", "overwrite", "False")
        self.dplug.plugins.customize.theme()

class Fore(wx.Frame):
    """ This Class Is Used to test colours for python files """
    def __init__(self, a, b, c, i):
        self.dplug = i
        self.gui = i.main
        wx.Frame.__init__(self, a, b, c)
        
        siz = wx.GridSizer(17,2,0,0)
        self.defname = wx.TextCtrl(self)
        self.identifier = wx.TextCtrl(self)
        self.classname = wx.TextCtrl(self)
        self.character = wx.TextCtrl(self)
        self.commentblock = wx.TextCtrl(self)
        self.commentline = wx.TextCtrl(self)
        self.decorator = wx.TextCtrl(self)
        self.default = wx.TextCtrl(self)
        self.number = wx.TextCtrl(self)
        self.string = wx.TextCtrl(self)
        self.stringeol = wx.TextCtrl(self)
        self.triple = wx.TextCtrl(self)
        self.tripledouble = wx.TextCtrl(self)
        self.word = wx.TextCtrl(self)
        self.word2 = wx.TextCtrl(self)
        self.operator = wx.TextCtrl(self)
        siz.Add(wx.StaticText(self, -1, "DEFNAME"))
        siz.Add(self.defname)
        siz.Add(wx.StaticText(self, -1, "IDENTIFIER"))
        siz.Add(self.identifier)
        siz.Add(wx.StaticText(self, -1, "CLASSNAME"))
        siz.Add(self.classname)
        siz.Add(wx.StaticText(self, -1, "CHARACTER"))
        siz.Add(self.character)
        siz.Add(wx.StaticText(self, -1, "COMMENTBLOCK"))
        siz.Add(self.commentblock)
        siz.Add(wx.StaticText(self, -1, "COMMENTLINE"))
        siz.Add(self.commentline)
        siz.Add(wx.StaticText(self, -1, "DECORATOR"))
        siz.Add(self.decorator)
        siz.Add(wx.StaticText(self, -1, "DEFAULT"))
        siz.Add(self.default)
        siz.Add(wx.StaticText(self, -1, "NUMBER"))
        siz.Add(self.number)
        siz.Add(wx.StaticText(self, -1, "STRING"))
        siz.Add(self.string)
        siz.Add(wx.StaticText(self, -1, "STRINGEOL"))
        siz.Add(self.stringeol)
        siz.Add(wx.StaticText(self, -1, "TRIPLE"))
        siz.Add(self.triple)
        siz.Add(wx.StaticText(self, -1, "TRIPLEDOUBLE"))
        siz.Add(self.tripledouble)
        siz.Add(wx.StaticText(self, -1, "WORD"))
        siz.Add(self.word)
        siz.Add(wx.StaticText(self, -1, "WORD2"))
        siz.Add(self.word2)
        siz.Add(wx.StaticText(self, -1, "operator"))
        siz.Add(self.operator)
        but = wx.Button(self, -1, "Save")
        siz.Add(but)
        self.SetSizer(siz)
        
        self.Show()
        
        self.Bind(wx.EVT_BUTTON, lambda event: self.ok(), but)
    
    def ok(self):
        tab = self.gui.get_active_tab()
        tab.editor.StyleSetForeground(wx.stc.STC_P_DEFNAME, self.defname.GetValue())
        tab.editor.StyleSetForeground(wx.stc.STC_P_IDENTIFIER, self.identifier.GetValue())
        tab.editor.StyleSetForeground(wx.stc.STC_P_CLASSNAME, self.classname.GetValue())
        tab.editor.StyleSetForeground(wx.stc.STC_P_CHARACTER, self.character.GetValue())
        tab.editor.StyleSetForeground(wx.stc.STC_P_COMMENTBLOCK, self.commentblock.GetValue())
        tab.editor.StyleSetForeground(wx.stc.STC_P_COMMENTLINE, self.commentline.GetValue())
        tab.editor.StyleSetForeground(wx.stc.STC_P_DECORATOR, self.decorator.GetValue())
        tab.editor.StyleSetForeground(wx.stc.STC_P_DEFAULT, self.default.GetValue())
        tab.editor.StyleSetForeground(wx.stc.STC_P_NUMBER, self.number.GetValue())
        tab.editor.StyleSetForeground(wx.stc.STC_P_STRING, self.string.GetValue())
        tab.editor.StyleSetForeground(wx.stc.STC_P_STRINGEOL, self.stringeol.GetValue())
        tab.editor.StyleSetForeground(wx.stc.STC_P_TRIPLE, self.triple.GetValue())
        tab.editor.StyleSetForeground(wx.stc.STC_P_TRIPLEDOUBLE, self.tripledouble.GetValue())
        tab.editor.StyleSetForeground(wx.stc.STC_P_WORD, self.word.GetValue())
        tab.editor.StyleSetForeground(wx.stc.STC_P_OPERATOR, self.operator.GetValue())
        tab.editor.StyleSetForeground(wx.stc.STC_P_WORD2, self.word2.GetValue())
